/*
 *  Copyright (c) 1992-2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

// the required Matisse C++ header
#include "matisseCXX.h"
// the headers for schema classes
#include "Person.h"
#include "Employee.h"

using namespace std;

// namespaces containing Matisse classes
using namespace matisse;
using namespace matisse::reflect;

// namespaces containing the Schema classes
using namespace examples::cxx_examples::chaps_4_5;

// A simple example which shows how to use generated code for user classes.
int main(int argc, char **argv)
{
  if (argc < 3)
  {
    cerr << "Usage: " << argv[0] << " <host> <db>" << endl;
    return -1;
  }
  try
  {
    // The third argument is given so that the connection object can find
    // the mapping between the C++ class Person, which is defined in the 
    // "examples::cxx_examples::chaps_4_5" namespace and the schema class
    //  Person defined in the "examples.cxx_examples.chaps_4_5" namespace.
    // MtDynamicObjectFactory *factory = new MtDynamicObjectFactory("examples::cxx_examples::chaps_4_5", "examples.cxx_examples.chaps_4_5");
    // MtDatabase db(argv[1], argv[2], factory);
    //
    // In this example, there is a direct namespace mapping between 
    // the C++ namespace and the Schema namespace, so the default factory can handle it
    MtDatabase db(argv[1], argv[2]);

    // open, select and start access to the database
    db.open();
    db.startTransaction();

    // create a new Person
    Person& p = Person::create(db);
    // modify attributes
    p.setFirstName("John");
    p.setLastName("Smith");
    p.setAge(42);

    // create a new Employee
    Employee& e = Employee::create(db);
    // set attributes
    e.setFirstName("Jane");
    e.setLastName("Jones");
    // age is nullable, so leave unset

    // create timestamp from string representation
    matisse::MtTimestamp ts("2002-01-01");
    e.setHireDate(ts);
    matisse::MtNumeric num(1000000.00);
    e.setSalary(num);

    // list all Persons
    std::cout << std::endl << Person::getInstanceNumber(db)
              << " Persons in the database" << std::endl;
    // open an iterator on all the instances of Person and all
    // subclasses; if you wish to exclude subclasses, use
    // Person::ownInstanceIterator() instead
    MtObjectIterator<Person> piter = Person::instanceIterator(db);
    while (piter.hasNext())
    {
      // use object reference
      Person &x = piter.next();
      // show attributes and name of class
      std::cout << "\t" << x.getFirstName() << " " << x.getLastName()
                << " is a " << x.getMtClass().getMtName() << std::endl;
    }

    db.commit();

    db.startTransaction();

    // remove created objects
    std::cout << "\nRemoving created objects" << std::endl;
    p.remove();
    e.remove();

    // list again to show deletion, similar to above
    std::cout << "\nAfter deletion:" << std::endl;
    std::cout << Person::getInstanceNumber(db)
              << " Persons in the database" << std::endl;
    MtObjectIterator<Person> piter2 = Person::instanceIterator(db);
    while (piter2.hasNext())
    {
      Person &x = piter2.next();
      std::cout << "\t" << x.getFirstName() << " " << x.getLastName()
                << " is a " << x.getMtClass().getMtName() << std::endl;
    }

    db.commit();

    db.close();
  }
  catch (MtException &e)
  {
    std::cerr << e << std::endl;
  }
  catch (...)
  {
    std::cerr << "Unknown exception" << std::endl;
  }
  return 0;
}
