/*
 *  Copyright (c) 1992-2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

// the required Matisse C++ header
#include "matisseCXX.h"
// the schema class
#include "Person.h"

// namespaces containing Matisse classes
using namespace matisse;
using namespace matisse::reflect;

// namespaces containing the Schema classes
using namespace examples::cxx_examples::chaps_6_7_8;

// Sample application which uses an entry-point dictionary associated
// with the Person::comment attribute. The commentDict is a full text
// dictionary so every word in the comment attribute will provide an
// entry point. (see examples.odl for specification)
int main(int ac, char **av)
{
  if (ac < 4)
  {
    std::cerr << "Usage: " << av[0] << " <host> <db> <searchString>"
              << std::endl;
    return -1;
  }
  try
  {
    // In this example, there is a direct namespace mapping between 
    // the C++ namespace and the Schema namespace, so the default factory can handle it
    MtDatabase db(av[1], av[2]);

    db.open();

    // add some Person objects
    db.startTransaction();
    Person& p1 = Person::create(db);
    p1.setFirstName("John");
    p1.setLastName("Jones");
    p1.setComment("weak needs have always hindered reality");
    Person& p2 = Person::create(db);
    p2.setFirstName("Fred");
    p2.setLastName("Jones");
    p2.setComment("in reality weak knees hindered many");

    std::cout << "\nAdding 2 objects with comments:" << std::endl
              << "   " << p1.getFirstName() << " " << p1.getLastName()
              << " says \"" << p1.getComment() << "\"" << std::endl
              << "   " << p2.getFirstName() << " " << p2.getLastName()
              << " says \"" << p2.getComment() << "\"" << std::endl;

    db.commit();

    db.startVersionAccess();


    // set the search string from command line
    std::string searchString = av[3];
    int hits = 0;

    std::cout << "\nLooking for : " << searchString << std::endl;
    // open an iterator on the number of Persons that match
    MtObjectIterator<Person> pIter = Person::commentDictIterator(db,
                                                                 searchString);
    while (pIter.hasNext())
    {
      Person &p = pIter.next();
      std::cout << "\t" << p.getFirstName() << " " << p.getLastName()
                << std::endl;
      hits++;
    }
    std::cout << db << ":  " << hits << " comment fields contained \""
              << searchString << "\"" << std::endl;
        
    MtSize num = Person::getCommentDictDictionary(db).getObjectNumber(searchString);
    std::cout << "\n" << num << " matching object(s)" << std::endl;

    db.endVersionAccess();

    db.startTransaction();

    std::cout << "\nRemoving objects..." << std::endl;
    p1.remove();
    p2.remove();
    db.commit();
          
    db.close();
  }
  catch (MtException &e)
  {
    std::cerr << e << std::endl;
  }
  catch (...)
  {
    std::cerr << "Unknown exception" << std::endl;
  }
  return 0;
}       

