/*
 *  Copyright (c) 1992-2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

// the required Matisse C++ header
#include "matisseCXX.h"
// the headers for schema classes
#include "Person.h"
#include "Employee.h"
#include "Manager.h"

using namespace std;

// namespaces containing Matisse classes
using namespace matisse;
using namespace matisse::reflect;

// namespaces containing the Schema classes
using namespace examples::cxx_examples::multidbs;

// this example shows how to access with the same application 2 logical databases
// into the database physical database server.

void createObjects(char *host, char *dbname, char *dbnamespace, int minId)
{
  std::cout << "=========== Create Objects ===========" << std::endl;
  try
  {
    MtDynamicObjectFactory *factory = new MtDynamicObjectFactory("examples::cxx_examples::multidbs", dbnamespace);
    MtDatabase db(host, dbname, factory);

    // open, select and start access to the database
    db.open();
    db.startTransaction();

    // create a new Person
    Person& p = Person::create(db);
    p.setId(minId+1);
    p.setFirstName("John");
    p.setLastName("Smith");
    p.setAge(42);

    // create a new Employee
    Employee& e = Employee::create(db);
    e.setId(minId+2);
    e.setFirstName("Jane");
    e.setLastName("Jones");
    matisse::MtTimestamp ts1("2011-01-01");
    e.setHireDate(ts1);
    matisse::MtNumeric num1(3000.00);
    e.setSalary(num1);

    // create a new Manager
    Manager& m = Manager::create(db);
    m.setId(minId+3);
    m.setFirstName("Elmo");
    m.setLastName("Tuttle");
    m.setComment("kill two birds with one stone");
    m.setGender("Male");
    matisse::MtTimestamp ts2("2011-11-01");
    m.setHireDate(ts2);
    matisse::MtNumeric num2(5000.00);
    m.setSalary(num2);
    m.setReportsTo(m);

    // Set a relationship 
    e.setReportsTo(m);

    // list all Persons
    std::cout << std::endl << Person::getInstanceNumber(db)
              << " Persons in the database" << std::endl;
    // open an iterator on all the instances of Person and all
    // subclasses; if you wish to exclude subclasses, use
    // Person::ownInstanceIterator() instead
    MtObjectIterator<Person> piter = Person::instanceIterator(db);
    while (piter.hasNext())
    {
      // use object reference
      Person &x = piter.next();
      // show attributes and name of class
      std::cout << "\t" << x.getFirstName() << " " << x.getLastName()
                << " is a " << x.getMtClass().getMtFullName() << std::endl;
    }

    db.commit();

    db.close();
  }
  catch (MtException &e)
  {
    std::cerr << e << std::endl;
  }
  catch (...)
  {
    std::cerr << "Unknown exception" << std::endl;
  }
}

void deleteObjects(char *host, char *dbname, char *dbnamespace)
{
  std::cout << "=========== Delete Objects ===========" << std::endl;
  try
  {
    MtDynamicObjectFactory *factory = new MtDynamicObjectFactory("examples::cxx_examples::multidbs", dbnamespace);
    MtDatabase db(host, dbname, factory);

    // open, select and start access to the database
    db.open();
    db.startTransaction();

    // remove created objects
    std::cout << "\nRemoving created objects" << std::endl;
    //p.remove();
    //e.remove();

    // list again to show deletion, similar to above
    std::cout << "\nAfter deletion:" << std::endl;
    std::cout << Person::getInstanceNumber(db)
              << " Persons in the database" << std::endl;
    MtObjectIterator<Person> piter2 = Person::instanceIterator(db);
    while (piter2.hasNext())
    {
      Person &x = piter2.next();
      std::cout << "\t" << x.getFirstName() << " " << x.getLastName()
                << " is a " << x.getMtClass().getMtName() << std::endl;
    }

    db.commit();

    db.close();
  }
  catch (MtException &e)
  {
    std::cerr << e << std::endl;
  }
  catch (...)
  {
    std::cerr << "Unknown exception" << std::endl;
  }
}

int main(int argc, char **argv)
{
  if (argc < 3)
  {
    cerr << "Usage: " << argv[0] << " <host> <db>" << endl;
    return -1;
  }

  createObjects(argv[1], argv[2], "examples.cxx_examples.multidbs.db_one", 1000);

  createObjects(argv[1], argv[2], "examples.cxx_examples.multidbs.db_two", 2000);


  return 0;
}
